# Tests

To automatize testing, name each test file as: `test_XXX.c`. All `test_XXX.c` files must be in the `src/` directory.

Use the command `make` to compile each source files. They will be placed in the `build/` directory created by the command.

To run all the tests, use: `make run`.

NB: If you have implemented the functions of `maze.h` directly in your `test_XXX.c` file, the compilation will fail. To solve the definition conflicts, you have to delete the functions from your `test_XXX.c` file and add at the beginning of your file: `include "maze.h"`.

## How to redefined a function from a header file in a source file?

If you need to redefined a function from a header file in your source file (for instance, to count the number of times the function is called), without having to redefined all the functions that calls it, you must add at the beginning of your source file the following:

```c
#define function_name_in_header_file(arg1, arg2, ..., argN) function_name_in_source_file(arg1, arg2, ..., argN)
#include "header.h"
#undef function_name_in_header_file
```

It will replace all the instances of `function_name_in_header_file` by `function_name_in_source_file` in the code included from `header.h`.

### Example

To count how many times the function `add_wall()` is called by `generate_maze_aux()`, I need to redefined the function in my source file to increment, for instance, a global variable each time a wall is added to the maze. To do so, I create a function called `test_add_wall()` with the same signature that the original `add_wall()` function.

Then, I implement my new function. In this example, I just need to copy-paste the original function algorithm and I add the code to increment my global variable into it.

For the `generate_maze_aux()` function to call my `test_add_wall()` function, I need to add at the beginning of my source file:

```c
#define add_wall(graph, i, j, width, size) test_add_wall(graph, i, j, width, size)
#include "maze.h"
#undef add_wall
```

That's it!
