#ifndef QUEUE_H
#define QUEUE_H

#include <stdbool.h>
#include <stdlib.h>

/**
 * @brief Structure to represent a queue.
 * 
 * This queue uses an array to store the elements. It keeps track of the front,
 * rear, size, and capacity to handle enqueue and dequeue operations efficiently.
 */
typedef struct queue queue_t;
struct queue {
    int *data;       ///< Array to hold the data in the queue.
    int front;       ///< Index of the front of the queue.
    int rear;        ///< Index of the rear of the queue.
    int size;        ///< Number of elements currently in the queue.
    int capacity;    ///< Maximum capacity of the queue.
};

/**
 * @brief Function to initialize the queue.
 * 
 * Initializes a queue with the given capacity, setting all necessary variables.
 * 
 * @param capacity The maximum number of elements the queue can hold.
 * @return A pointer to the newly created queue.
 */
queue_t *createQueue(int capacity);

/**
 * @brief Function to add an element to the queue.
 * 
 * Adds an item to the rear of the queue. The queue resizes if necessary.
 * 
 * @param q The queue to add the element to.
 * @param item The item to add to the queue.
 */
void enqueue(queue_t *q, int item);

/**
 * @brief Function to remove an element from the queue.
 * 
 * Removes the element at the front of the queue.
 * 
 * @param q The queue to remove the element from.
 * @return The item that was removed from the front of the queue.
 */
int dequeue(queue_t *q);

/**
 * @brief Function to check if the queue is empty.
 * 
 * Checks if the queue has any elements.
 * 
 * @param q The queue to check.
 * @return True if the queue is empty, false otherwise.
 */
bool isEmpty(queue_t *q);

#endif // QUEUE_H
