#include "maze.h"
#include "test.h"
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h> 

#define NB_LOOPS 30
#define MAX_EXPONENT 15
#define BASE_2 2

#define TEN_E3 (unsigned long long int) 1e3
#define TEN_E6 (unsigned long long int) 1e6
#define TEN_E9 (unsigned long long int) 1e9

void check_error(int const error_code, char const *const msg)
{
	if (error_code == -1)
	{
		perror(msg);
		exit(EXIT_FAILURE);
	}
}

unsigned int power(unsigned int const x, unsigned int const n)
{
	unsigned int result = 1;
	
	for (unsigned int i = 0; i < n; i++)
		result *= x;

	return result;
}

void clear_maze(struct graph *const graph)
{
	if (graph == NULL || graph->vertices == NULL)
		return;

	unsigned long long int size = graph->width * graph->height;

	for (unsigned long long int i = 0; i < size; i++)
		graph->vertices[i].parent = NULL;
}

unsigned long long int substract_positive(long long int x, long long int y)
{
	return (x >= y) ? x - y : y - x;
}

int main(int argc, char const *argv[])
{
	if (argc != 2)
	{
		fprintf(stderr, "Usage: %s output_file\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	char const *const filename = argv[1];

//	int const fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644);
	int const fd = open(filename, O_WRONLY | O_CREAT | O_APPEND, 0644);
	check_error(fd, "open");

//	dprintf(fd, "SIZE;TIME\n");

	struct timespec start, end;

	srand(time(NULL));

	struct graph *graph = NULL;
	long long int size;
	unsigned long long int s, ns;
	long double ms;

//	unsigned int base = 2;

	for (int i = 4194; i <= 1e4; i++)
	{
//		size = power(base, i);
		size = i;
		printf("Size of the maze: %llu\n", size * size);

		printf("Maze initialization...\n");
		graph = init_graph(size, size);

		if (graph == NULL)
		{
			print_failure("Failed to init graph\n");
			check_error(close(fd), "close");
			exit(EXIT_FAILURE);
		}

		printf("Maze generation...\n");
		generate_maze(graph);
		s = 0;
		ns = 0;

		printf("Performance test in progress\n");
		for (int j = 0; j < NB_LOOPS; j++)
		{
			clock_gettime(CLOCK_MONOTONIC, &start);
			find_exit(graph, 0);
			clock_gettime(CLOCK_MONOTONIC, &end);

			s += substract_positive(end.tv_sec, start.tv_sec);
			ns += substract_positive(end.tv_nsec, start.tv_nsec);

			s += ns / TEN_E9;
			ns = ns % TEN_E9;

			clear_maze(graph);
		}
		free_graph(graph);
		printf("\n");

		ms = s * TEN_E3 / ((long double) NB_LOOPS) + ns / ((long double) NB_LOOPS * TEN_E6);

		dprintf(fd, "%llu;%.6Lf\n", size * size, ms);
	}

	check_error(close(fd), "close");

	return 0;
}