#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "maze.h"
#include "queue.h"

// Structure to represent an edge
struct edge {
    int parent;
    int child;
};

struct edge *bfs_tree(struct graph *graph, int width, int height, int start, int *num_edges) {
    //Pré-condition : start ne doit pas être la valeur de la sortie 
    //Exemple : 24 est la sortie, donc on ne doit pas mettre 24 en argument
    
    // Vérifie si le graphe est valide
    if (graph == NULL || graph->vertices == NULL) {
        return NULL;
    }

    // Créer un tableau pour garder une trace des sommets visités
    int *visited = (int *)calloc(width * height, sizeof(int));
    if (visited == NULL) {
        return NULL;  // Échec de l'allocation
    }

    // Créer une file d'attente pour BFS
    struct vertex **queue = (struct vertex **)malloc(width * height * sizeof(struct vertex *));
    int front = 0, rear = 0;

    // Initialiser la file d'attente avec le sommet de départ
    struct vertex *start_vertex = &graph->vertices[start];
    queue[rear++] = start_vertex;
    visited[start] = 1;

    // Variable pour garder une trace des arêtes du BFS
    struct edge *edges = (struct edge *)malloc(width * height * sizeof(struct edge));
    int edge_count = 0;

    // BFS
    while (front < rear) {
        struct vertex *current = queue[front++];

        // Récupère les voisins du sommet courant
        struct vertex **neighbours = get_neighbours(current);
        for (int i = 0; neighbours[i] != NULL; i++) {
            struct vertex *neighbour = neighbours[i];

            // Si le voisin n'a pas été visité, on l'ajoute à la file d'attente
            if (!visited[neighbour->id]) {
                visited[neighbour->id] = 1;
                queue[rear++] = neighbour;

                // Enregistrer l'arête en utilisant 'parent' et 'child'
                edges[edge_count].parent = current->id;
                edges[edge_count].child = neighbour->id;
                edge_count++;
            }
        }
    }

    // Nettoyer la mémoire
    free(visited);
    free(queue);

    *num_edges = edge_count;

    return edges;  // Retourne les arêtes du BFS
}



// Function to print the spanning tree
void print_bfs_tree(struct edge *tree, int num_edges, int root) {
    printf("Spanning tree (BFS):\n");

    queue_t *q = createQueue(num_edges);
    enqueue(q, root);

    while (!isEmpty(q)) {
        int parent = dequeue(q);
        printf("Node %d: ", parent);

        // Traverse the children of the current parent node
        for (int i = 0; i < num_edges; i++) {
            if (tree[i].parent == parent) {
                printf("%d ", tree[i].child);
                enqueue(q, tree[i].child);
            }
        }
        printf("\n"); // New line for each parent after listing its children
    }

    free(q->data);
    free(q);
}

int main() {
    int width = 5;
    int height = 5;
    int start_id = 0;

    // Initialiser le graphe (maze)
    struct graph *graph = init_graph(width, height);  // La fonction retourne un pointeur vers un struct graph, pas un struct vertex

    if (graph == NULL) {
        printf("Échec de l'initialisation du graphe.\n");
        return 1;
    }

    // Générer le labyrinthe
    generate_maze(graph);  // On passe simplement le graphe, sans largeur ni hauteur

    int num_edges;
    struct edge *tree = bfs_tree(graph, width, height, start_id, &num_edges);
    print_bfs_tree(tree, num_edges, start_id);


    // Libérer le graphe
    free_graph(graph);  // La fonction attend un graphe, pas un sommet

    return 0;
}

