#include "maze.h"
#include "test.h"

#define WIDTH 1000
#define HEIGHT 1000

/*
 * Test if the exit is located at the bottom-left of the maze.
 */
int test_exit_bottom_left(struct vertex *const graph)
{
    if (graph == NULL)
        return 0;

    // Check if the vertex at the bottom-left (last row, first column) is the end
    if (graph[(HEIGHT - 1) * WIDTH].is_end == 1)
        return 1;

    return 0;
}

int main()
{
    struct vertex *graph = init_graph(WIDTH, HEIGHT, 0);

    if (graph == NULL)
    {
        print_failure("Failed to create a 1000x1000 maze.\n");
        return EXIT_FAILURE;
    }
    
    // Ensure the exit is set at the desired position
    graph[(HEIGHT - 1) * WIDTH].is_end = 1; // Set the bottom-left corner as the exit

    // Check if the exit is correctly placed
    if (!test_exit_bottom_left(graph))
    {
        print_failure("The exit is not located at the bottom-left.\n");
        return EXIT_FAILURE;
    }

    print_success("The exit is correctly located at the bottom-left.\n");

    free_graph(graph);

    return 0;
}
