#include "maze.h"
#include "test.h"
#include <time.h>

#define WIDTH 10
#define HEIGHT 10

// Fonction pour vérifier si le chemin est correct
int verify_path(struct graph *const graph, struct vertex *path) {
    if (path == NULL) {
        return 0;  // Pas de chemin trouvé
    }

    unsigned int const width = graph->width;
    unsigned int const height = graph->height;
    struct vertex *cur_vertex = path;

    // Vérifie que chaque vertex dans le chemin pointe correctement vers son parent
    while (cur_vertex->parent != NULL) {
        // Le parent doit être un voisin valide du vertex actuel (top, bottom, left, right)
        struct vertex *parent = cur_vertex->parent;

        // Vérification de la validité du voisinage
        if (parent->id == cur_vertex->id - 1 && parent->top == cur_vertex)  // parent à gauche
            ;
        else if (parent->id == cur_vertex->id + 1 && parent->right == cur_vertex)  // parent à droite
            ;
        else if (parent->id == cur_vertex->id - width && parent->bottom == cur_vertex)  // parent en haut
            ;
        else if (parent->id == cur_vertex->id + width && parent->top == cur_vertex)  // parent en bas
            ;
        else {
            return 0;  // Parent invalide
        }
        cur_vertex = parent;
    }

    // Vérifier que l'on est bien arrivé à la sortie
    return (cur_vertex->is_end == 1);
}

/*
int main() {
    srand(time(NULL));

    struct graph *graph = init_graph(WIDTH, HEIGHT);

    if (graph == NULL) {
        print_failure("Failed to create 10x10 maze.\n");
        return EXIT_FAILURE;
    }

    // Générer le labyrinthe
    generate_maze(graph);
    
    // Afficher le labyrinthe généré
    display_maze(graph);

    // Trouver la sortie et afficher le chemin
    struct vertex *path = find_exit(graph, 0);
    display_path(graph, path);

    // Vérification du chemin
    if (verify_path(graph, path)) {
        print_success("Path is correct!\n");
    } else {
        print_failure("Path is incorrect!\n");
    }

    // Libérer la mémoire allouée
    free_graph(graph);

    return EXIT_SUCCESS;
}
*/
