// Replace add_wall() in maze.h by test_add_wall()
#define add_wall(graph, i, j) test_add_wall(graph, i, j)
#include "maze.h"
#undef add_wall // Undo the previous modification

#include "test.h"

#define WIDTH 10
#define HEIGHT 1

/*
 * Preconditions: None.
 * Postconditions: None.
 *
 * Returns 1 if the width and height of the graph are equals to 1.
 * Otherwise, it returns 0.
 */
int test_is_n_x_one(struct graph *const graph)
{
	if (graph == NULL)
		return 0;

	unsigned int is_n_x_one = 1;
	unsigned int size = WIDTH * HEIGHT;

	for (unsigned int i = 0; i < size && is_n_x_one == 1; i++)
	{
		is_n_x_one = is_n_x_one && graph->vertices[i].top == NULL && graph->vertices[i].bottom == NULL;

		if (i == 0)
			is_n_x_one = is_n_x_one && graph->vertices[i].left == NULL;
		else
			is_n_x_one = is_n_x_one && graph->vertices[i].left == &graph->vertices[i-1];

		if (i == size - 1)
			is_n_x_one = is_n_x_one && graph->vertices[i].right == NULL;
		else
			is_n_x_one = is_n_x_one && graph->vertices[i].right == &graph->vertices[i+1];			
	}

	return is_n_x_one;
}

unsigned int nb_walls_added = 0;

/*
 * Postconditions: - nb_walls_added is incremented by one each time a wall
 *                 is added to the graph->
 */
void test_add_wall(struct graph *const graph, int const i, int const j)
{
	int const width = graph->width;
	int const height = graph->height;
	int const size = width * height;

	if (graph == NULL || i < 0 || j < 0 || i >= size || j >= size)
		return;

	// If graph->vertices[j] is at the right of graph->vertices[i]
	// and graph->vertices[i] is not a right border cell
	if (i + 1 == j && i % width != width-1)
	{
		graph->vertices[i].right = NULL;
		graph->vertices[j].left = NULL;
		nb_walls_added++;
	}
	// Else if graph->vertices[j] is at the bottom of graph->vertices[i]
	else if (i + width == j)
	{
		graph->vertices[i].bottom = NULL;
		graph->vertices[j].top = NULL;
		nb_walls_added++;
	}
	// Else if graph->vertices[j] is at the left of graph->vertices[i]
	// and graph->vertices[i] is not a left border cell
	else if (i - 1 == j && i % width != 0)
	{
		graph->vertices[i].left = NULL;
		graph->vertices[j].right = NULL;
		nb_walls_added++;
	}
	// Else if graph->vertices[j] is at the top of graph->vertices[i]
	else if (i - width == j)
	{
		graph->vertices[i].top = NULL;
		graph->vertices[j].bottom = NULL;
		nb_walls_added++;
	}

	// Else graph->vertices[i] and graph->vertices[j] are not connected,
	// so no wall can separate them.
}

/*
 * Preconditions: None.
 * Postconditions: graph represents a maze randomly generated, or NULL.
 *
 * Returns 1 if the function add_wall() is never called.
 * Otherwise, it returns 0.
 */
int test_do_not_create_wall(struct graph *const graph)
{
	generate_maze(graph);

	return nb_walls_added == 0;
}

int main()
{
	struct graph *graph = init_graph(WIDTH, HEIGHT);

	if (graph == NULL)
	{
		print_failure("Failed to create a 10x1 maze.\n");
		return EXIT_FAILURE;
	}

	if (!test_is_n_x_one(graph))
	{
		print_failure("The maze is not a 10x1 maze.\n");
		return EXIT_FAILURE;
	}

	print_success("The maze is a 10x1 maze.\n");

	if (!test_do_not_create_wall(graph))
	{
		print_failure("Walls have been created in the 10x1 maze.\n");
		return EXIT_FAILURE;
	}

	print_success("No wall has been created in the 10x1 maze.\n");

	free_graph(graph);

	return 0;
}