#include <stdio.h>
#include <time.h>
#include "maze.h"
#include <linux/time.h>

// Fonction pour mesurer le temps écoulé en millisecondes
double get_time_diff(struct timespec start, struct timespec end) {
    return (end.tv_sec - start.tv_sec) * 1000.0 + (end.tv_nsec - start.tv_nsec) / 1000000.0;
}

// Teste les performances des fonctions generate_maze et find_exit
void test_performance(int width, int height) {
    struct timespec start, end;
    struct graph *maze;

    // Mesure du temps pour la génération du labyrinthe
    clock_gettime(CLOCK_MONOTONIC, &start);
    maze = init_graph(width, height);
    if (maze == NULL) {
        printf("Erreur lors de l'initialisation du labyrinthe\n");
        return;
    }
    generate_maze(maze);
    clock_gettime(CLOCK_MONOTONIC, &end);
    double generate_time = get_time_diff(start, end);

    // Mesure du temps pour trouver la sortie
    clock_gettime(CLOCK_MONOTONIC, &start);
    struct vertex *exit_vertex = find_exit(maze, 0);
    clock_gettime(CLOCK_MONOTONIC, &end);
    double find_exit_time = get_time_diff(start, end);

    // Affiche les résultats
    printf("Dimensions du labyrinthe : %dx%d\n", width, height);
    printf("Temps pour générer le labyrinthe : %.3f ms\n", generate_time);
    printf("Temps pour trouver la sortie : %.3f ms\n", find_exit_time);
    
    // Libération de la mémoire
    free_graph(maze);
}

int main() {
    // Différentes tailles de labyrinthe pour observer la complexité
    int sizes[][2] = {{10, 10}, {20, 20}, {30, 30}, {40, 40}, {50, 50}};
    int num_sizes = sizeof(sizes) / sizeof(sizes[0]);

    for (int i = 0; i < num_sizes; i++) {
        int width = sizes[i][0];
        int height = sizes[i][1];
        test_performance(width, height);
    }

    return 0;
}
