#include "maze.h"
#include "test.h"
#include <time.h>
#include "colors.h"


#define WIDTH 10
#define HEIGHT 10

void display_maze(struct graph *const graph) {
    if (graph == NULL || graph->vertices == NULL)
    	return;

    unsigned int const width = graph->width;
    unsigned int const height = graph->height;

    for (unsigned int j = 0; j < 3 * height; j++)
    {
    	for (unsigned int i = 0; i < width; i++)
    	{
    		if (j % 3 == 0)
    		{
    			printf("+%s+",
    				(graph->vertices[i + (j / 3) * width].top == NULL) ? "---" : "   ");
    		}
    		else if (j % 3 == 1)
    		{
    				printf("%c   %c",
    					(graph->vertices[i + (j / 3) * width].left == NULL) ? '|' : ' ',
    					(graph->vertices[i + (j / 3) * width].right == NULL) ? '|' : ' ');
    		}
    		else
    		{

    			printf("+%s+",
    				(graph->vertices[i + (j / 3) * width].bottom == NULL) ? "---" : "   ");
    		}
    	}
    	printf("\n");
    }
}

#define CURSOR_UP(POS) printf("\x1b[%dA", POS)
#define CURSOR_DOWN(POS) printf("\x1b[%dB", POS)
#define CURSOR_HORIZONTAL_ABSOLUTE(POS) printf("\x1b[%dG", POS)

void display_path(struct graph *const graph, struct vertex *path)
{
	if (graph == NULL || path == NULL)
		return;

	unsigned int const width = graph->width;
	unsigned int const height = graph->height;

	while (path != path->parent)
	{
		CURSOR_UP(3 * (height - path->id / width - 1) + 2);
		CURSOR_HORIZONTAL_ABSOLUTE(5 * (path->id % width + 1) - 2);
		printf(ANSI_COLOR_GOLD"#"ANSI_COLOR_RESET);
		CURSOR_HORIZONTAL_ABSOLUTE(-5 * (path->id % width + 1) + 2);
		CURSOR_DOWN(3 * (height - path->id / width - 1) + 2);
		path = path->parent;
	}
	CURSOR_UP(3 * (height - path->id / width - 1) + 2);
	CURSOR_HORIZONTAL_ABSOLUTE(5 * (path->id % width + 1) - 2);
	printf("%s#%s",ANSI_COLOR_GOLD,ANSI_COLOR_RESET);
	CURSOR_HORIZONTAL_ABSOLUTE(-5 * (path->id % width + 1) + 2);
	CURSOR_DOWN(3 * (height - path->id / width - 1) + 2);
}

int main()
{
	srand(time(NULL));

	struct graph *graph = init_graph(WIDTH, HEIGHT);

	if (graph == NULL)
	{
		print_failure("Failed to create 10x10 maze.\n");
		return EXIT_FAILURE;
	}

	generate_maze(graph);
	display_maze(graph);

	struct vertex *path = find_exit(graph, 0);
	display_path(graph, path);
/*
	struct vertex *cur_vertex = path;
	while (cur_vertex != cur_vertex->parent)
	{
		printf("%3u <- ", cur_vertex->id);
		cur_vertex = cur_vertex->parent;
	}
	printf("%3u\n", cur_vertex->id);
*/
	free_graph(graph);

	return EXIT_SUCCESS;
}