#define generate_maze_aux(graph, first_vertex_id, cur_width, cur_height) \
		test_generate_maze_aux(graph, first_vertex_id, cur_width, cur_height)
#include "maze.h"
#undef generate_maze_aux

#include "test.h"

#define WIDTH 1000
#define HEIGHT 1000

unsigned long int nb_walls = 0;
unsigned long int nb_doors = 0;

/*
 * Postconditions: - nb_walls is incremented by one each time a full wall has been created.
 *                 - nb_doors is incremented by one each time a door is left in a full wall.
 */
void test_generate_maze_aux(struct graph *const graph, int const first_vertex_id,
	int const cur_width, int const cur_height)
{
	if (cur_width == 1 || cur_height == 1)
		return;

	unsigned int const width = graph->width;

	if (cur_width >= cur_height)
	{
		int const r_vertex_id = random_int(first_vertex_id, first_vertex_id + cur_width - 2, 1);
		int const r_door_id = random_int(r_vertex_id, r_vertex_id + width * (cur_height - 1), width);
		int vertex_i;

		for (int i = 0; i < cur_height; i++)
		{
			vertex_i = r_vertex_id + width * i;

			if (vertex_i != r_door_id)
				add_wall(graph, vertex_i, vertex_i + 1);
			else
				nb_doors++;
		}
		nb_walls++;

		int const left_width = r_vertex_id - first_vertex_id + 1;
		test_generate_maze_aux(graph, first_vertex_id, left_width, cur_height);
		test_generate_maze_aux(graph, r_vertex_id + 1, cur_width - left_width, cur_height);
	}
	else
	{
		int const r_vertex_id = random_int(first_vertex_id,
									width * (cur_height - 1 + first_vertex_id / width) - 1, width);
		int const r_door_id = random_int(r_vertex_id, r_vertex_id + cur_width - 1, 1);
		int vertex_i;

		for (int i = 0; i < cur_width; i++)
		{
			vertex_i = r_vertex_id + i;

			if (vertex_i != r_door_id)
				add_wall(graph, vertex_i, vertex_i + width);
			else
				nb_doors++;
		}
		nb_walls++;

		int const top_height = (r_vertex_id - first_vertex_id) / width + 1;
		test_generate_maze_aux(graph, first_vertex_id, cur_width, top_height);
		test_generate_maze_aux(graph, r_vertex_id + width, cur_width, cur_height - top_height);
	}
}

/*
 * Preconditions: None.
 * Postconditions: graph represents a maze randomly generated, or NULL.
 *
 * Returns 1 if each wall has a door. Otherwise, it returns 0.
 */
int test_walls_have_a_door(struct graph *const graph)
{
	generate_maze(graph);

	return nb_walls == nb_doors;
}

int main()
{
	struct graph *graph = init_graph(WIDTH, HEIGHT);

	if (graph == NULL)
	{
		print_failure("Failed to create a 1000x1000 maze.\n");
		return EXIT_FAILURE;
	}

	if (!test_walls_have_a_door(graph))
	{
		print_failure("Not all walls have a door. The maze may not be connected.\n");
		return EXIT_FAILURE;
	}

	print_success("All walls have a door. The maze is connected.\n");

	free_graph(graph);

	return 0;
}